/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@parc.xerox.com
				chiba@is.s.u-tokyo.ac.jp
*/

/*
  WrapperClass generates a wrapper function for each member function.
  Every call on the member function is replaced with the call on
  the wrapper function.  You can define a subclass of WrapperClass
  so that the compiler generates wrapper functions that you want.
  The default wrapper function simply calls the wrapped function.

  The wrapper function is generated by MakeWrapper().  You can override
  this function.  Otherwise, you can just override WrapperBody(), which
  is invoked by MakeWrapper().  WrapperBody() returns the function body
  of the wrapper.  See wrapper.cc.
*/

#include "mop.h"

class WrapperClass : public Class {
public:
    WrapperClass(Ptree* d, Ptree* m) : Class(d, m) {}

    Ptree* TranslateBody(Environment*, Ptree*);
    virtual Ptree* WrapperName(Ptree* name);

    Ptree* TranslateMemberFunctionBody(Environment*, Ptree*, BOOL, Ptree*);
    virtual void MakeWrapper(Environment*, Ptree*, Ptree*, TypeInfo&, BOOL);
    virtual Ptree* WrapperBody(Environment*, Ptree*, Ptree*, int, TypeInfo&);

    Ptree* TranslateMemberCall(Environment*, Ptree*, Ptree*, Ptree*,
			       Ptree*);
    Ptree* TranslateMemberCall(Environment*, Ptree*, Ptree*);
};
