/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#include <string.h>
#include <stdlib.h>
#include "buffer.h"

Program::~Program()
{
    delete replacement;
}

char Program::Get()
{
    if(buf[index] == '\0')
	return buf[index];
    else
	return buf[index++];
}

ProgramFile::ProgramFile(FILE* fptr)
: Program()
{
    const Size = 8192;
    int c;
    uint i, s;
    char* buffer;

    s = Size;
    buffer = new char[s];
    i = 0;
    while((c = fgetc(fptr)) != EOF){
	buffer[i++] = c;
	if(i >= s){
	    char* new_buf = new char[s + Size];
	    bcopy(buffer, new_buf, i);
	    delete [] buffer;
	    buffer = new_buf;
	    s += Size;
	}
    }

    buffer[i++] = '\0';
    buf = buffer;
    size = s;
    index = 0;
}

ProgramFile::~ProgramFile()
{
    delete [] buf;
    buf = nil;
}

ProgramFromStdin::ProgramFromStdin()
: Program()
{
    const Size = 256 * 1024;
    buf = new char[Size];
    buf_size = Size;
    index = size = 0;
}

ProgramFromStdin::~ProgramFromStdin()
{
    delete [] buf;
    buf = nil;
}

char ProgramFromStdin::Get()
{
    if(size >= buf_size){
	cerr << "ProgramFromStdin: sorry, out of memory\n";
	exit(1);
    }

    if(index >= size){
	int c = fgetc(stdin);
	if(c == EOF)
	    c = '\0';

	buf[size++] = c;
    }

    if(buf[index] == '\0')
	return buf[index];
    else
	return buf[index++];
}

// class StringBuffer

const Quantum = 4;
const BufSize = 1 << Quantum;

ProgramString::ProgramString()
: Program()
{
    buf = new char[BufSize];
    buf[0] = '\0';
    size = BufSize;
    index = str_length = 0;
}

ProgramString::~ProgramString()
{
    delete [] buf;
    buf = nil;
}

ProgramString& ProgramString::operator << (const char* str)
{
    int len = strlen(str) + 1;

    if(str_length + len < size){
	bcopy((char*)str, &buf[str_length], len);
	str_length += len - 1;
    }
    else{
	size = (str_length + len + BufSize) & ~(BufSize - 1);
	char* newbuf = new char[size];
	bcopy(buf, newbuf, str_length);
	bcopy((char*)str, &newbuf[str_length], len);
	delete [] buf;
	buf = newbuf;
	str_length += len - 1;
    }

    return *this;
}

ProgramString& ProgramString::operator << (const char c)
{
    if(str_length + 2 < size){
	buf[str_length] = c;
	buf[++str_length] = '\0';
    }
    else{
	size = (str_length + 2 + BufSize) & ~(BufSize - 1);
	char* newbuf = new char[size];
	bcopy(buf, newbuf, str_length);
	newbuf[str_length] = c;
	newbuf[++str_length] = '\0';
	delete [] buf;
	buf = newbuf;
    }

    return *this;
}
