/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#ifndef _function_h
#define _function_h

#include "ptree-core.h"

template <class T> class mopArray {
public:
    mopArray(int s = 8) {
	num = 0;
	size = s;
	array = new T[s];
    }
    ~mopArray() { delete [] array; }
    int Number() { return num; }
    T& operator [] (uint index) { return Ref(index); }
    T& Ref(uint i) { return array[i]; }
    void Append(T& v) {
	if(num >= size){
	    size += 8;
	    T* a = new T[size];
	    bcopy((char*)array, (char*)a, num * sizeof(T));
	    delete [] array;
	    array = a;
	}

	array[num++] = v;
    }

private:
    uint num, size;
    T* array;
};

class TemplateFunction {
public:
    TemplateFunction(Ptree* def);
    Ptree* TranslateSelf(Environment*);
    Ptree* TranslateFuncall(Environment*, Environment*, Ptree*);

private:
    BOOL BindAllTypeVariables(Environment*, Environment*, Ptree*,
			      Ptree*, Ptree*&);
    void BindTypeVariable(Environment*, TypeInfo&, TypeInfo&);
    Ptree* MakeInstance(Environment*, Ptree*, Ptree*);

protected:
    Ptree*	definition;
    PtreeArray	arguments;
};

#endif /* _function_h */
