/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

//  Hash Table

#ifndef _hash_h
#define _hash_h

typedef void* HashValue;

class HashTableEntry;

class HashTable {
public:
    HashTable();
    HashTable(int) {}
    void MakeTable();
    ~HashTable();
    void DeleteAll();
    int IsEmpty();
    int AddEntry(char* key, HashValue value, int* index = 0);
    int AddEntry(char* key, int len, HashValue value, int* index = 0);
    int Lookup(char* key, HashValue* value);
    int Lookup(char* key, int len, HashValue* value);
    HashValue Peek(int index);
    int RemoveEntry(char* key);
    int RemoveEntry(char* key, int len);
    void ReplaceValue(int index, HashValue value);

protected:
    int Lookup2(char* key, HashValue* val, int* index);
    int Lookup2(char* key, int len, HashValue* val, int* index);
    unsigned int StringToInt(char*);
    unsigned int StringToInt(char*, int);
    int HashFunc(unsigned int p, int n);

protected:
    HashTableEntry	*entries;
    int			Size;	// the max number of entries.
				// should be a prime number
    int			Prime2;
};

class BigHashTable : public HashTable {
public:
    BigHashTable();
};

extern char* KeyString(char* key);
extern char* KeyString(char* key, int len);
extern char* ConcatKeyString(char* key1, char* key2, char* key3);
inline void DeleteKeyString(char* key) { delete [] key; }

inline char* DuplicateString(char* str) {
    return(str == 0 ? str : KeyString(str));
}

#endif /* _hash_h */
