/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#ifndef _mop_h
#define _mop_h

#include "ptree-core.h"
#include "typeinfo.h"
#include "binding.h"

class Class {
public:
    Class(Ptree* definition, Ptree* meta_arguments);

// introspection

    Ptree* Name();
    Ptree* BaseClasses();
    Ptree* Members();
    Ptree* Definition() { return definition; }
    virtual char* MetaclassName();	// automaticallly implemented
					// by Metaclass

    Class* NthBaseClass(Environment*, int n);
    Ptree* NthMemberName(int);
    int IsMember(Ptree*);
    BOOL LookupMemberType(Environment*, Ptree*, TypeInfo&);

// translation

    virtual Ptree* TranslateSelf(Environment*);
    virtual Ptree* TranslateClassName(Environment*, Ptree*, Ptree*);
    virtual Ptree* TranslateBaseClasses(Environment*, Ptree*);
    virtual Ptree* TranslateBody(Environment*, Ptree*);

    virtual Ptree* TranslateMemberFunctionBody(Environment*, Ptree*, BOOL,
					       Ptree*);

    virtual Ptree* TranslateInitializer(Environment*, Ptree*,
					PtreeArray&, PtreeArray&);

    virtual Ptree* TranslateAssign(Environment*, Ptree*, Ptree*, Ptree*);
    virtual Ptree* TranslateFunctionCall(Environment*, Ptree*, Ptree*);
    virtual Ptree* TranslateSubscript(Environment*, Ptree*, Ptree*);
    virtual Ptree* TranslatePostfix(Environment*, Ptree*, Ptree*);
    virtual Ptree* TranslateUnary(Environment*, Ptree*, Ptree*);
    virtual Ptree* TranslateBinary(Environment*, Ptree*, Ptree*, Ptree*);

    virtual Ptree* TranslateNew(Environment*, Ptree*, Ptree*, Ptree*, Ptree*,
				Ptree*);
    virtual Ptree* TranslateMemberRead(Environment*, Ptree*, Ptree*, Ptree*);
    virtual Ptree* TranslateMemberRead(Environment*, Ptree*);
    virtual Ptree* TranslateMemberWrite(Environment*, Ptree*, Ptree*, Ptree*,
					Ptree*, Ptree*);
    virtual Ptree* TranslateMemberWrite(Environment*, Ptree*, Ptree*, Ptree*);
    virtual Ptree* TranslateMemberCall(Environment*, Ptree*, Ptree*, Ptree*,
				       Ptree*);
    virtual Ptree* TranslateMemberCall(Environment*, Ptree*, Ptree*);

    virtual Ptree* TranslateUserStatement(Environment*, Ptree*, Ptree*,
					  Ptree*, Ptree*);
    virtual Ptree* TranslateStaticUserStatement(Environment*, Ptree*, Ptree*);

// others

    virtual BOOL AcceptTemplate();

    static BOOL Initialize();
    virtual Ptree* Finalize();

    static Ptree* TranslateNewType(Environment* env, Ptree* type_name);
    static Ptree* TranslateArgumentList(Environment* env, Ptree* expr);
    static Ptree* TranslateIndex(Environment* env, Ptree* index);
    static Ptree* TranslateExpression(Environment* env, Ptree* expr);
    static Ptree* TranslateExpression(Environment* env, Ptree* expr,
				      TypeInfo&);
    static Ptree* TranslateStatement(Environment* env, Ptree* expr);
    static void InsertBeforeToplevel(Ptree*);
    static void AppendAfterToplevel(Ptree*);

    static void RegisterNewModifier(char*);
    static void RegisterNewAccessSpecifier(char*);
    static void RegisterNewWhileStatement(char*);
    static void RegisterNewForStatement(char*);
    static void RegisterNewClosureStatement(char*);
    static void RegisterMetaclass(char* keyword, char* class_name);

private:
    static BOOL IsArglist(Ptree*);

    Ptree* definition;
    Class* next;
    static Class* class_list;

friend class Token;
};


// not documented class --- internal use only

typedef Class* (*opcxx_MetaclassCreator)(Ptree*, Ptree*);

class opcxx_ListOfMetaclass {
public:
    opcxx_ListOfMetaclass(char*, opcxx_MetaclassCreator, BOOL);
    static Class* New(Ptree*, Ptree*, Ptree*);
    static Class* New(char*, Ptree*, Ptree*);
    static BOOL AlreadyRecorded(char*);
    static void PrintAllMetaclasses();

private:
    opcxx_ListOfMetaclass* next;
    char* name;
    opcxx_MetaclassCreator proc;
    static opcxx_ListOfMetaclass* head;
};

#endif  /* _mop_h */
