/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#include <string.h>
#include "ptree.h"
#include "quote-class.h"
#include "buffer.h"

// MetaclassName() should be automatically generated but we write it
// down here for bootstrapping.

char* QuoteClass::MetaclassName()
{
    return "QuoteClass";
}

Ptree* QuoteClass::TranslateMemberCall(Environment* env,
				       Ptree* member, Ptree* args)
{
    Ptree* name = SimpleName(member);
    char* str;

    if(Ptree::Eq(name, "qMake")){
	Ptree* arg1 = Ptree::First(Ptree::Second(args));
	if(arg1->Reify(str) && str != nil)
	    return ProcessBackQuote(FALSE, str);
	else
	    ErrorMessage("bad argument for qMake()", arg1);
    }
    if(Ptree::Eq(name, "qMakeStatement")){
	Ptree* arg1 = Ptree::First(Ptree::Second(args));
	if(arg1->Reify(str) && str != nil)
	    return ProcessBackQuote(TRUE, str);
	else
	    ErrorMessage("bad argument for qMakeStatement()", arg1);
    }

    return Class::TranslateMemberCall(env, member, args);
}

Ptree* QuoteClass::SimpleName(Ptree* qualified_name)
{
    if(qualified_name->IsLeaf())
	return qualified_name;
    else
	return Ptree::First(Ptree::Last(qualified_name));
}

Ptree* QuoteClass::ProcessBackQuote(BOOL is_statement, char* str)
{
    ProgramString result;

    if(is_statement)
	result << "PtreeHead::Statement(";

    result << "(Ptree*)(PtreeHead()";
    while(*str != '\0')
	if(*str == '`'){
	    result << '+';
	    while(*++str != '`')
		if(*str != '\0')
		    result << *str;
		else{
		    ErrorMessage("unmatched backquote for Ptree::qMake()"
				 " or qMakeStatement()");
		    break;
		}

	    ++str;
	}
	else{
	    result << "+\"";
	    while(*str != '`' && *str != '\0')
		result << *str++;

	    result << '"';
	}

    result << ')';
    if(is_statement)
	result << ')';

    char* rstr = (char*)result.Read(0);
    return new UserLeaf(rstr, strlen(rstr));
}
