/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#ifndef _syntax_h
#define _syntax_h

#include "types.h"
#include "typeinfo.h"

class Ptree;
class NameInfo;
class Environment;
class TypeInfo;
class Binding;
class ClassTypeInfo;
union TokenVal;
class Class;

class Syntax {
public:
    static Ptree* Constructor(Ptree*, TokenVal&, Ptree*, TokenVal&,
			      Ptree*, Ptree* = nil, Ptree* = nil);
    static Ptree* QualifiedConstructor(Ptree*, TokenVal&, Ptree*, TokenVal&,
				       Ptree*, Ptree* = nil, Ptree* = nil);
    static Ptree* QualifiedConstructorHead(Ptree*, Ptree*);
    static void Destructor(Ptree*);
    static Ptree* OldFuncDeclaration(Ptree*, Ptree*);
    static Ptree* FuncDeclaration(Ptree*, Ptree*, Ptree*);
    static Ptree* Declaration(Ptree*, Ptree*, Ptree*);
    static Ptree* CastOperator(Ptree*);
    static void IsNotQualifiedClassName(Ptree*);
    static Ptree* DeclaredName(Ptree*);
    static Ptree* DeclaredName(Ptree*, BOOL&);
    static Ptree* TypedefName(Ptree*);
    static Ptree* DeclaredTemplateName(Ptree*);
    static Ptree* MakeAssignExpr(Ptree*, TokenVal&, Ptree*);
    static Ptree* MakeFuncallExpr(Ptree*, TokenVal&, Ptree*, TokenVal&);

    static Ptree* TranslateTempFuncall(TypeInfo&, Environment*, Ptree*);

    static TypeInfoId IsMemberName(Ptree*, Environment*, TypeInfo&, Class*&);
    static TypeInfoId LookupMemberType(Ptree*, TypeInfo&, TypeInfo&);
    static BOOL LookupType(Ptree*, Environment*, BOOL&, TypeInfo&);
    static void AnalyzeConstType(Ptree*, Environment*, TypeInfo&);
    static void AnalyzeCastType(Ptree*, Environment*, TypeInfo&);
    static void AnalyzeFstyleCast(Ptree*, Environment*, TypeInfo&);
    static void AnalyzeQClass(Ptree*, Environment*, TypeInfo&);
    static void DereferencedType(TypeInfo&, TypeInfo&);
    static ClassTypeInfo* LookupClassOfQname(Ptree* qname, Environment*);
    static BOOL AnalyzeNthArgType(TypeInfo&, int, Environment*, TypeInfo&);
    static Binding* BindQualifiedName(Environment*, Ptree*, Binding** = nil);

private:
    static void SetTypeInfo(Binding*, Environment*, BOOL&, TypeInfo&);
    static void SetPseudoClassTypeInfo(Environment*, TypeInfo&);
    static void AnalyzeVariableType(NameInfo*, Environment*, TypeInfo&);
    static void AnalyzeVariableType(NameInfo*, Ptree*, Environment*,
					TypeInfo&);
    static void AnalyzeTypespecifier(Binding*, Environment*, TypeInfo&);
    static void ResolveTypedefName(Ptree*, Environment*, TypeInfo&);
    static int DereferencedType2(Ptree*, int);
    static Ptree* GetArgList(TypeInfo&);
    static int GetArgList2(Ptree*, int);
};

#endif /* _syntax_h */
