/*
  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.

  Author: Shigeru Chiba		chiba@is.s.u-tokyo.ac.jp
*/

#ifndef _typeinfo_h
#define _typeinfo_h

#include "types.h"

enum TypeInfoId {
    UndefType,
    BuiltInType, ClassType, EnumType, TemplateType,
    PointerType, ReferenceType, PointerToMemberType,
    ArrayType, FunctionType, TemplateFunctionType,

    // These are used by class Binding
    VariableName, FunctionName, TypedefType, ThisVariable,
    ImmutableVarName, DataMemberName, MemberFunctionName,
    TypeVariable,
};

// These are used by TypeInfo::WhatBuiltinType()
//
enum {
    CharType = 1, IntType = 2, ShortType = 4, LongType = 8,
    SignedType = 16, UnsignedType = 32, FloatType = 64,
    DoubleType = 128, VoidType = 256,
};

class Binding;
class Environment;
class Class;
class Ptree;

class TypeInfo {
public:
    TypeInfo(TypeInfoId, Binding*, Environment*);
    TypeInfo() { binding = nil; full_typename = nil; }
    TyepInfo(TypeInfo& t) { Set(t); }
    ~TypeInfo();
    TypeInfoId WhatIs() { return (TypeInfoId)kind; }
    BOOL IsUndef() { return BOOL(kind == UndefType); }
    void Undef() { Set(UndefType, nil, nil); }
    void BeBoolean() { Set(BuiltInType, nil, nil); }
    void BeInt() { Set(BuiltInType, nil, nil); }
    void BeVoid() { Set(UndefType, nil, nil); }

    void Reference();
    void Reference(TypeInfo&);
    void Dereference();
    void Dereference(TypeInfo&);
    void DereferenceIfRefType();
    BOOL NthArgument(int, TypeInfo&);
    Class* ClassMetaobject();
    uint IsBuiltInType();
    Ptree* FullTypeName();

    Ptree* MakePtree(Ptree* = nil);

private:
    void Set(TypeInfoId k, Binding* b, Environment* e, uint level = 0);
    void Set(TypeInfo&);
    int DerefLevel() { return deref_level; }
    Binding* GetBinding() { return binding; }
    Environment* Env() { return env; }
    Environment* ClassEnvironment();
    void SetFullTypeName(Ptree* p) { full_typename = p; }
    Ptree* MakeFunctionTypePtree(Ptree*);

    short	   deref_level;
    short	   ref_level;
    unsigned short kind;
    unsigned short original_kind;
    Binding*	   binding;	// these may be nil if BuiltInType
    Environment*   env;		// see BeBoolean(), BeInt(), and
				// Syntax::AnalyzeConstType().
				// binding may be nil if the type is
				// a template argument.
    Ptree*	   full_typename;

    static Ptree* star;
    static Ptree* ampersand;
    static Ptree* comma;
    static Ptree* dots;
    static Ptree* brackets;
    static Ptree* left_paren;
    static Ptree* right_paren;

    friend class Syntax;
};

#endif /* _typeinfo_h */
